import bs
import weakref
import bsUtils

_gFactory = None

class Factory(object):
    """Media and other shared data for making instances"""
    def __init__(self):
        self.flagMaterial = bs._newMaterial()
        bs._materialAddAction(self.flagMaterial,
                             conditions=( ("srcYoungerThan",100),'and',
                                          ("dstIsMaterial",bs.objectMaterial)),
                             actions=( ("nodeMod","collide",False)))
        bs._materialAddAction(self.flagMaterial,
                             conditions=("dstIsMaterial",bs.footingMaterial),
                             actions=(("message","self","atConnect","footing",1),
                                      ("message","self","atDisconnect","footing",-1)))
        self.impactSounds = bs.getSound("metalHit");
        self.skidSounds = bs.getSound("metalSkid");
        bs._materialAddAction(self.flagMaterial,
                             conditions=("dstIsMaterial",bs.footingMaterial),
                             actions=(("impactSound",self.impactSounds,2,5),
                                      ("skidSound",self.skidSounds,2,5)))

        self.noHitMaterial = bs._newMaterial()
        bs._materialAddAction(self.noHitMaterial,
                             conditions=(("dstIsMaterial",bs.pickupMaterial),'or',
                                         ("dstIsMaterial",bs.attackMaterial)),
                             actions=(("partMod","collide",False)))

        # we also dont want anything moving it
        bs._materialAddAction(self.noHitMaterial,
                             conditions=(("dstIsMaterial",bs.objectMaterial),'or',
                                         ("dstNotMaterial",bs.footingMaterial)),
                             actions=(("partMod","collide",False),
                                      ("partMod","physical",False)))
        
        self.flagTexture = bs.getTexture("flagColor")


def getFactory():
    """ return the global factory object, initializing it if necessary """
    global _gFactory
    if _gFactory == None:
        _gFactory = Factory()
    return _gFactory

class FlagPickedUpMessage(object):
    """A flag has been picked up."""
    def __init__(self,flag,node):
        self.flag = flag
        self.node = node

class FlagDeathMessage(object):
    """A flag has died."""
    def __init__(self,flag):
        self.flag = flag

class FlagDroppedMessage(object):
    """A flag has been dropped."""
    def __init__(self,flag,node):
        self.flag = flag
        self.node = node

class Flag(bs.Actor):
    def __init__(self,activity,retain,translate=(0,1,0),color=(1,1,1),materials=(),touchable=True,droppedTimeout=None):
        bs.Actor.__init__(self,activity,retain)

        self._initialPosition = None
        self._hasMoved = False

        factory = getFactory()

        if not touchable: materials = (factory.noHitMaterial,)+materials

        self.node = bs.newNode("flag",
                               owner=activity,
                               attrs={'translate':(translate[0],translate[1]+0.75,translate[2]),
                                      'colorTexture':factory.flagTexture,
                                      'color':color,
                                      'materials':(bs.objectMaterial,factory.flagMaterial)+materials},
                               delegate=self)

        self._droppedTimeout = droppedTimeout
        if self._droppedTimeout is not None:
            self._count = self._droppedTimeout
            self._tickTimer = bs.Timer(1000,call=bs.WeakCall(self._tick),repeat=True)
            self._counter = bs.newNode('text',owner=self.node,attrs={'inWorld':True,'color':(1,1,1,0.7),'scale':[0.015],'hAlign':'center'},name='CTF Flag Text')
        else:
            self._counter = None

        self._heldCount = 0

    def _tick(self):
        if self.node.exists():
            # grab our initial position after one tick (in case we fall)
            if self._initialPosition is None:
                self._initialPosition = self.node.translate
            # keep track of when we first move; we don't count down until then
            if not self._hasMoved:
                t = self.node.translate
                if max(abs(t[i] - self._initialPosition[i]) for i in range(3)) > 1.0:
                    self._hasMoved = True

            if self._heldCount > 0 or not self._hasMoved:
                self._count = self._droppedTimeout
                self._counter.text = ''
            else:
                self._count -= 1
                if self._count <= 10:
                    t = self.node.translate
                    self._counter.translate = (t[0],t[1]+1.3,t[2])
                    self._counter.text = str(self._count)
                    if self._count < 1: self.handleMessage(bs.DieMessage())
                else:
                    self._counter.text = ''
                    
    def _hideScoreText(self):
        bsUtils.animate(self._scoreText,'scale',{0:self._scoreText.scale[0],200:0})

    def setScoreText(self,t):
        """ utility func to show a message over our flag; handy for scores """
        if not self.node.exists(): return
        try: exists = self._scoreText.exists()
        except Exception: exists = False
        if not exists:
            startScale = 0.0
            m = bs.newNode('math',owner=self.node,attrs={'input1':(0,1.4,0),'operation':'add'})
            bs.connectAttr(self.node,'translate',m,'input2')
            self._scoreText = bs.newNode('text',
                                          owner=self.node,
                                          attrs={'text':t,
                                                 'inWorld':True,
                                                 'scale':[0.02],
                                                 'hAlign':'center'},
                                          name='spaz score txt')
            bs.connectAttr(m,'output',self._scoreText,'translate')
        else:
            startScale = self._scoreText.scale[0]
            self._scoreText.text = t
        self._scoreText.color = self.node.color
        bsUtils.animate(self._scoreText,'scale',{0:startScale,200:0.02})
        self._scoreTextHideTimer = bs.Timer(1000,bs.WeakCall(self._hideScoreText))

    def handleMessage(self,m):
        if isinstance(m,bs.DieMessage):
            if self.node.exists():
                bs.deleteNode(self.node)
                a = self.getActivity()
                if a: a.handleMessage(FlagDeathMessage(self))
        elif isinstance(m,bs.HitMessage):
            bs.nodeMessage(self.node,"impulse",m.pos[0],m.pos[1],m.pos[2],
                           m.velocity[0],m.velocity[1],m.velocity[2],
                           m.magnitude,m.velocityMagnitude,m.radius,0,
                           m.forceDirection[0],m.forceDirection[1],m.forceDirection[2])
        elif isinstance(m,bs.OutOfBoundsMessage):
            # we just kill ourselves when out-of-bounds.. would we ever not want this?..
            self.handleMessage(bs.DieMessage(how='fall'))
        elif isinstance(m,bs.PickedUpMessage):
            self._heldCount += 1
            if self._heldCount == 1 and self._counter is not None:
                self._counter.text = ''
            a = self.getActivity()
            if a: a.handleMessage(FlagPickedUpMessage(self,m.node))
        elif isinstance(m,bs.DroppedMessage):
            self._heldCount -= 1
            if self._heldCount < 0:
                print 'Flag held count < 0'
                self._heldCount = 0
            a = self.getActivity()
            if a: a.handleMessage(FlagDroppedMessage(self,m.node))
        else:
            bs.Actor.handleMessage(self,m)

